class Admin::LabsController < OrbitMemberController
  include Admin::PersonalLabsHelper
  layout "member_plugin"

  before_action :set_lab, only: [:show, :edit , :update, :destroy]
  before_action :set_plugin
  before_action :get_settings,:only => [:new, :edit, :setting]

  before_action :need_access_right
  before_action :allow_admin_only, :only => [:index, :setting]

  def index
    @labs = Lab.sort_year.page(params[:page]).per(10)
  end

  def new
    @member = MemberProfile.find_by(:uid=>params['uid'].to_s) rescue nil
    @lab = Lab.new
    if params[:desktop]
      render :layout => false
    end
  end
  def analysis_report
    role = params[:role_id]
    year_start = params[:year_start].to_i
    year_end = params[:year_end].to_i
    graph_by = params[:graph_by]

    @data = get_chart_data(year_start,year_end,role,params[:graph_by])

    render :layout => false
  end

  def download_excel
    year_start = params[:year_start].to_i
    year_end = params[:year_end].to_i
    @data = get_data_for_excel(year_start,year_end)
    respond_to do |format|
      format.xlsx {
        response.headers['Content-Disposition'] = 'attachment; filename="labs.xlsx"'
      }
    end
  end
  def create

    if !lab_params['member_profile_id'].blank?

      @member = MemberProfile.find(lab_params['member_profile_id']) rescue nil
      @lab = Lab.new(lab_params)
      @lab.save

      if params[:desktop] == "true"
        render json: {"data" => get_paper_list}.to_json
      else
        redirect_to params['referer_url']
      end

    elsif !params[:author_members].blank?

      params[:author_members].each do |author_member|

        lab_params['member_profile_id'] = author_member
        @lab = Lab.new(lab_params)
        @lab.save

        if params[:desktop] == "true"
          render json: {"data" => get_paper_list}.to_json
        end

      end

      redirect_to params['referer_url']

    else

      lab_params['member_profile_id'] = User.find(current_user.id).member_profile_id

      @lab = Lab.new(lab_params)
      @lab.save

      if params[:desktop] == "true"
        render json: {"data" => get_paper_list}.to_json
      end

      redirect_to params['referer_url']

    end
    
  end

  def edit
    @member = @lab.member_profile rescue nil
    if params[:desktop]
      render :layout => false
    end
  end

  def update
    @member = @lab.member_profile rescue nil
    @lab.update_attributes(lab_params)
    @lab.save
    if params[:desktop] == "true"
      render json: {"data" => get_paper_list}.to_json
    else
      redirect_to params['referer_url']
    end
  end

  def destroy
    @lab.destroy
    respond_to do |format|
      format.html { redirect_to(admin_labs_url) }
      # format.xml { head :ok }
      format.js
      format.json {render json: {"success" => true}}
      end
  end

  def excel_format
    respond_to do |format|
      format.xlsx {
        response.headers['Content-Disposition'] = 'attachment; filename="labs_format.xlsx"'
      }
    end
  end

  def import_from_excel
    workbook = RubyXL::Parser.parse(params["import_file"].tempfile)
    sheet = workbook[0]
    if sheet.count <= 503
      sheet.each_with_index do |row, i|
        next if i < 3
        user = User.where(:user_name => row.cells[0].value).first rescue nil
        if !user.nil?
          mp = user.member_profile
          import_this_lab(row,mp)
        end
      end
      redirect_to admin_labs_url
    else
      redirect_to admin_labs_url(:error => "1")
    end
  end


  def toggle_hide
    if params[:ids]
      @labs = Lab.any_in(_id: params[:ids])

      @labs.each do |lab|
        lab.is_hidden = params[:disable]
        lab.save
      end
    end

    render json: {"success"=>true}
  end

  def setting
  end

  def frontend_setting
    @member = MemberProfile.find_by(:uid=>params['uid'].to_s) rescue nil
    @intro = LabIntro.find_by(:member_profile_id=>@member.id) rescue nil
    @intro = @intro.nil? ? LabIntro.new({:member_profile_id=>@member.id}) : @intro
  end

  def update_frontend_setting
    @member = MemberProfile.find(intro_params['member_profile_id']) rescue nil
    @intro = LabIntro.find_by(:member_profile_id=>@member.id) rescue nil
    @intro = @intro.nil? ? LabIntro.new({:member_profile_id=>@member.id}) : @intro
    @intro.update_attributes(intro_params)
    @intro.save
    redirect_to URI.encode('/admin/members/'+@member.to_param+'/Lab')
  end

  def get_settings
  end

  def set_plugin
    @plugin = OrbitApp::Plugin::Registration.all.select{|plugin| plugin.app_name.eql? 'Lab'}.first
  end

  private

  def set_lab
    path = request.path.split('/')
    if path.last.include? '-'
      uid = path[-1].split("-").last
      uid = uid.split("?").first
    else
      uid = path[-2].split("-").last
      uid = uid.split("?").first
    end
     @lab = Lab.find_by(:uid => uid) rescue Lab.find(params[:id])
  end

  def lab_params
    params.require(:lab).permit! rescue nil
  end

  def intro_params
    params.require(:lab_intro).permit! rescue nil
  end
end