class Lab
  include Mongoid::Document
  include Mongoid::Timestamps
  include OrbitModel::Status
  include Slug

  belongs_to :member_profile

  field :lab_title, :as=>:slug_title, localize: true
  field :location, localize: true
  field :participating_professor, localize: true
  field :participating_student, localize: true
  
  field :year
  field :language
  field :keywords
  field :extension_no
  field :research_direction
  field :facility
  field :url
  field :note
  field :create_user_id, :type => BSON::ObjectId
  field :update_user_id, :type => BSON::ObjectId

  # paginates_per 10
  
  has_many :lab_files, :autosave => true, :dependent => :destroy
  accepts_nested_attributes_for :lab_files, :allow_destroy => true

  before_validation :add_http

  index({year: -1, _id: -1}, { unique: false, background: false })
  scope :sort_year, ->{ order_by(:year => "desc", :id=>"desc") }
  scope :sort_for_frontend, ->{ where(:is_hidden=>false).order_by(:year => "desc", :id=>"desc") }

  def get_plugin_data(fields_to_show)
    plugin_datas = []
    fields_to_show.each do |field|
      plugin_data = self.get_plugin_field_data(field) rescue nil
      next if plugin_data.blank? or plugin_data['value'].blank?
      plugin_datas << plugin_data
    end
    plugin_datas
  end

  def self.get_plugin_datas_to_member(datas)

    fields_to_show = [
      "lab_title",
      "location"
    ]

    fields_to_remove = []

    pd_title = []

    fields_to_show.each do |t|
      if (self.fields[t].type.to_s == "String" rescue false)
        fields_to_remove << t if (datas.where(t.to_sym.ne => nil).or(t.to_sym.ne => "").count == 0 rescue false)
      else
        fields_to_remove << t if (datas.where(t.to_sym.ne => "").count == 0 rescue false)
      end
      pd_title <<  {
          "plugin_data_title" => I18n.t("personal_lab.#{t}")
        } if !fields_to_remove.include?(t)
    end

    fields_to_show = fields_to_show - fields_to_remove

    plugin_datas = datas.sort_for_frontend.collect do |p|

      pd_data = []
      fields_to_show.collect do |t|
        if t == "lab_title"
          link = OrbitHelper.url_to_plugin_show(p.to_param,'personal_lab')
          url_to_plugin_show_blank = OrbitHelper.instance_variable_get(:@url_to_plugin_show_blank)
          tmp_title = p.lab_title
          pd_data << { "data_title" => (url_to_plugin_show_blank ? tmp_title : "<a title=\"#{tmp_title}\" href=\"#{link}\" target=\"_blank\">#{tmp_title}</a>") }
        else
          pd_data << { "data_title" => p.send(t) }
        end
      end

      {
        "pd_datas" => pd_data
      }

    end

    return [pd_title,plugin_datas]

  end

  def get_plugin_field_data(field)
    case field
    when "language"
      value = I18n.t(self.language) rescue ""
    when "file"
      files = []
      self.lab_files.each do |lab_file|  
        url = lab_file.file.url
        title = (lab_file.title.blank? ? File.basename(lab_file.file.path) : lab_file.title)
        files << "<li><a href='#{url}'' target='_blank'>#{title}</li>"
      end
      value = files.join("")
    when "owner"
      path = OrbitHelper.url_to_plugin_show(self.member_profile.to_param, 'member') rescue '#'
      value = "<a href='#{path}'>#{self.member_profile.name}</a>"
    else
      value = self.send(field) rescue ""
    end

    value = (value =~ /\A#{URI::regexp(['http', 'https'])}\z/) ? "<a href='#{value}' target='blank'>#{value}</a>" : value
    
    {
      "key"=>field,
      "title_class"=>"lab-#{field.gsub('_','-')}-field",
      "value_class"=>"lab-#{field.gsub('_','-')}-value",
      "title"=>I18n.t('personal_lab.'+field),
      "value"=>value
    }
  end

  protected

  def add_http
    unless self.url.blank? || self.url[/^http:\/\//] || self.url[/^https:\/\//]
      self.url = 'http://' + self.url
    end
  end

end
